# -*- encoding: utf-8 -*-
import os
import yaml

from suite_py.lib.singleton import Singleton


class Config(metaclass=Singleton):
    _config = {}

    def __init__(self):
        pass

    def load(self):
        with open(
            os.path.join(os.environ["HOME"], ".suite_py/config.yml")
        ) as configfile:
            self._config = yaml.safe_load(configfile)

        self._config["user"]["projects_home"] = os.path.join(
            os.environ["HOME"], self._config["user"]["projects_home"]
        )

        if not self._config["user"].get("notify_channel"):
            self._config["user"]["notify_channel"] = "#review"

        if not self._config["user"].get("default_slug"):
            self._config["user"]["default_slug"] = "PRIMA-XXX"

        if not self._config["user"].get("captainhook_timeout"):
            self._config["user"]["captainhook_timeout"] = 30  # This is in seconds

        return self._config



# -*- encoding: utf-8 -*-
from distutils.version import StrictVersion
import time
import requests

from halo import Halo
from suite_py.lib.tokens import Tokens


tokens = Tokens()
baseurl = "https://drone-1.prima.it"


def get_last_build_url(repo):
    with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
        # necessario per far comparire la build che abbiamo appena pushato
        time.sleep(2)
        try:
            resp = requests.get(
                f"{baseurl}/api/repos/primait/{repo}/builds",
                headers={"Authorization": f"Bearer {tokens.drone}"},
            ).json()

            return f"{baseurl}/primait/{repo}/{resp[0]['number']}"
        except Exception:
            return ""


def get_pr_build_url(repo, commit_sha):
    with Halo(text="Contacting drone...", spinner="dots", color="magenta"):
        # necessario per far comparire la build che abbiamo appena pushato
        time.sleep(2)
        try:
            resp = requests.get(
                f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
                headers={"Authorization": f"Bearer {tokens.drone}"},
            ).json()
            build_number = list(
                filter(lambda build: build["after"] == commit_sha, resp)
            )[0]["number"]
            return f"{baseurl}/primait/{repo}/{build_number}"
        except Exception:
            return ""


def get_tag_from_builds(repo):
    tags = []
    builds = requests.get(
        f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
        headers={"Authorization": f"Bearer {tokens.drone}"},
    ).json()

    for build in builds:
        if build["event"] == "tag":
            tags.append(build["ref"].replace("refs/tags/", ""))

    tags = list(dict.fromkeys(tags))
    tags.sort(key=StrictVersion, reverse=True)
    return tags


def get_build_from_tag(repo, tag):
    builds = requests.get(
        f"{baseurl}/api/repos/primait/{repo}/builds?per_page=100",
        headers={"Authorization": f"Bearer {tokens.drone}"},
    ).json()

    for build in builds:
        if build["event"] == "tag":
            if build["ref"].replace("refs/tags/", "") == tag:
                return build["number"]
    return None


def launch_build(repo, build):
    return requests.post(
        f"{baseurl}/api/repos/primait/{repo}/builds/{build}",
        headers={"Authorization": f"Bearer {tokens.drone}"},
    ).json()



# -*- coding: utf-8 -*-
import sys
import requests

from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.logger import Logger

logger = Logger()


def entrypoint(project, action, timeout):
    lock = CaptainHook()
    if timeout:
        lock.set_timeout(timeout)
    if action in ["lock", "l"]:
        try:
            req = lock.lock_project(project)
            handle_request(req)
            logger.info(f"Bloccato deploy su staging del progetto {project}")
        except requests.exceptions.Timeout:
            logger.warning("Richiesta a Captainhook in timeout. Prova con suite-py --timeout=60 lock-project lock")
            sys.exit(1)
    elif action in ["unlock", "u"]:
        try:
            req = lock.unlock_project(project)
            handle_request(req)
            logger.info(f"Abilitato deploy su staging del progetto {project}")
        except requests.exceptions.Timeout:
            logger.warning("Richiesta a Captainhook in timeout. Prova con suite-py --timeout=60 lock-project unlock")
            sys.exit(1)
    else:
        logger.warning("Non ho capito che cosa devo fare")
        sys.exit(-1)


def handle_request(request):
    if request.status_code == 401:
        logger.error("Impossibie contattare captainhook, stai usando la VPN?")
        sys.exit(-1)
    if request.status_code == 500:
        logger.error(
            "Si è verificato un errore su captainhook. Chiedi ai devops di verificare (X_X)"
        )
        sys.exit(-1)
    if request.status_code != 200:
        logger.error("Qualcosa è andato storto durante la richiesta.")
        sys.exit(-1)

    return True



# -*- encoding: utf-8 -*-
import requests

from suite_py.lib.config import Config
from suite_py.lib.singleton import Singleton
from suite_py.lib.handler import git_handler as git


config = Config().load()


class CaptainHook(metaclass=Singleton):
    _baseurl = None
    _timeout = config["user"]["captainhook_timeout"]

    def __init__(self):
        self._baseurl = "http://captainhook-internal.prima.it"

    def lock_project(self, project):
        data = {"project": project, "status": "locked", "user": git.get_git_username()}
        return self.send_post_request("/projects/manage-lock", data)

    def unlock_project(self, project):
        data = {
            "project": project,
            "status": "unlocked",
            "user": git.get_git_username(),
        }
        return self.send_post_request("/projects/manage-lock", data)

    def status(self, project):
        return self.send_get_request(f"/projects/check?project={project}")

    def get_users_list(self):
        return self.send_get_request("/users/all")

    def send_post_request(self, endpoint, data):
        return requests.post(
            f"{self._baseurl}{endpoint}", data=data, timeout=self._timeout
        )

    def send_get_request(self, endpoint):
        return requests.get(f"{self._baseurl}{endpoint}", timeout=self._timeout)

    def set_timeout(self, timeout):
        self._timeout = timeout
